--[[
Copyright (C) Achimobil

Author: Achimobil
Date: 08.07.2022
Version: 0.1.0.0

Contact:
https://www.achimobil.de/

History:
V 0.1.0.0 @ 08.07.2022 - First Version.

Important:
Free for use in other mods - no permission needed, only provide my name.
No changes are to be made to this script without permission from Achimobil.

Frei verwendbar - keine erlaubnis nötig, Namensnennung im Mod erforderlich.
An diesem Skript dürfen ohne Genehmigung von Achimobil keine Änderungen vorgenommen werden.
]]

SiloObjectBaleTextureChangerSpecialization = {
    Version = "0.1.0.0",
    Name = "SiloObjectBaleTextureChangerSpecialization"
}
print(g_currentModName .. " - init " .. SiloObjectBaleTextureChangerSpecialization.Name .. "(Version: " .. SiloObjectBaleTextureChangerSpecialization.Version .. ")");

function SiloObjectBaleTextureChangerSpecialization.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(PlaceableSilo, specializations)
end

function SiloObjectBaleTextureChangerSpecialization.registerEventListeners(placeableType)
	SpecializationUtil.registerEventListener(placeableType, "onLoad", SiloObjectBaleTextureChangerSpecialization)
	SpecializationUtil.registerEventListener(placeableType, "onFinalizePlacement", SiloObjectBaleTextureChangerSpecialization)
	SpecializationUtil.registerEventListener(placeableType, "onPostFinalizePlacement", SiloObjectBaleTextureChangerSpecialization)
end

function SiloObjectBaleTextureChangerSpecialization.registerFunctions(placeableType)
	SpecializationUtil.registerFunction(placeableType, "updateObjectFillLevelsForTextures", SiloObjectBaleTextureChangerSpecialization.updateObjectFillLevelsForTextures)
end

function SiloObjectBaleTextureChangerSpecialization:onLoad(savegame)
-- print("SiloObjectBaleTextureChangerSpecialization:onLoad(savegame)");
	self.spec_siloObjectBaleTexture = {};
	local spec = self.spec_siloObjectBaleTexture;

	-- später vielleicht einstellbar, jetzt ist alles fix für mein Model
	spec.lastUsedFillTypeIndexForTexture = nil;
	
	-- used as parameter for search Texture
	spec.isRoundbale = false;
	spec.width = 1.2;
	spec.height = 0.9;
	spec.length = 2.2;
	
	-- used for identify items
	spec.i3dName = "squarebale220_vis";
	
	-- rootnode aus silo object fill level für jetzt
	spec.ChildNodeIds = {};
	local k = 0;
	while true do
		local siloObjectFillLevelKey = string.format("placeable.silo.siloObjectFillLevels.siloObjectFillLevel(%d)", k);
-- print("siloObjectFillLevelKey: " .. tostring(siloObjectFillLevelKey));
		if not self.xmlFile:hasProperty(siloObjectFillLevelKey) then
			break;
		end
		
		local rootNode = self.xmlFile:getValue(siloObjectFillLevelKey .. "#rootNode", nil, self.components, self.i3dMappings);
		
		-- get childs
		local numChildren = getNumOfChildren(rootNode);
-- print("numChildren: " .. tostring(numChildren));
		for i = 0, numChildren - 1 do
			local childNodeId = getChildAt(rootNode, i)
			if getName(childNodeId) == spec.i3dName then
				table.insert(spec.ChildNodeIds, childNodeId)
			end
			
			local numChildrenInner = getNumOfChildren(childNodeId)
-- print("numChildrenInner: " .. tostring(numChildrenInner));
			for j = 0, numChildrenInner - 1 do
				local childNodeIdInner = getChildAt(childNodeId, j)
				if getName(childNodeIdInner) == spec.i3dName then
					table.insert(spec.ChildNodeIds, childNodeIdInner)
				end
			end	
		end
		
		k = k + 1;
	end
	
	function spec.fillLevelChangedCallback(fillType, delta)
		self:updateObjectFillLevelsForTextures()
	end
end

function SiloObjectBaleTextureChangerSpecialization:onFinalizePlacement(savegame)
	local spec = self.spec_siloObjectBaleTexture;
	for _, sourceStorage in pairs(self.spec_silo.loadingStation:getSourceStorages()) do
		sourceStorage:addFillLevelChangedListeners(spec.fillLevelChangedCallback)
	end
end

function SiloObjectBaleTextureChangerSpecialization:onPostFinalizePlacement(savegame)
	self:updateObjectFillLevelsForTextures()
end

function SiloObjectBaleTextureChangerSpecialization:updateObjectFillLevelsForTextures()
	local spec = self.spec_siloObjectBaleTexture;

	local currentFillTypeIndex = nil;
	
	for fillTypeIndex, fillLevel in pairs(self:getFillLevels()) do
		if currentFillTypeIndex == nil and fillLevel > 1 then
			currentFillTypeIndex = fillTypeIndex;
		end
	end
	
	-- print("currentFillTypeIndex: " .. tostring(currentFillTypeIndex));
	
	if currentFillTypeIndex ~= nil and currentFillTypeIndex ~= spec.lastUsedFillTypeIndexForTexture then
		-- print("Change from: " .. tostring(spec.lastUsedFillTypeIndexForTexture));
		-- print("Change to: " .. tostring(currentFillTypeIndex));
			
		-- ich muss die ballen auslesen die möglich sind mit deren xml file ein mal am anfang und dann kann ich bei änderng des filltypes diese werte nutzen für was geht rein in das lager
		-- vielleicht auch einfach nur alles wo es einen 240er ballen für gibt? Lager dann auf diese einstellen mit gesamtmenge
		-- local baleXMLFilename = g_baleManager:getBaleXMLFilename(fillTypeId, isRoundbale, width, height, length, diameter, customEnvironment)
		local baleXMLFilename = g_baleManager:getBaleXMLFilename(currentFillTypeIndex, spec.isRoundbale, spec.width, spec.height, spec.length, nil, nil)
		-- print("baleXMLFilename: " .. tostring(baleXMLFilename))
		local xmlFileBale = XMLFile.load("TempBale", baleXMLFilename, g_baleManager.baleXMLSchema)
		local _, baseDirectory = Utils.getModNameAndBaseDirectory(baleXMLFilename)
		-- print("baseDirectory: " .. tostring(baseDirectory))
		local fillTypes = {}
		Bale.loadFillTypesFromXML(fillTypes, xmlFileBale, baseDirectory)

		local supportsWrapping = 0;
		for _, fillTypeInfo in pairs(fillTypes) do
			if fillTypeInfo.fermenting ~= nil and fillTypeInfo.fermenting.outputFillTypeIndex == currentFillTypeIndex and fillTypeInfo.fermenting.requiresWrapping then
				supportsWrapping = 1;
			end
		end
		-- print("supportsWrapping: " .. tostring(supportsWrapping));
		for childIndex, childNodeId in pairs(spec.ChildNodeIds) do
			-- textur auf ballen legen
			Bale.setFillTypeTextures(childNodeId, fillTypes, currentFillTypeIndex);
			-- wrapping oder nicht
			setShaderParameter(childNodeId, "wrappingState", supportsWrapping, 0, 0, 0, false)
		end
		
		spec.lastUsedFillTypeIndexForTexture = currentFillTypeIndex;
	end
end